/***********************************************************************

	Copyright (c) 1994 Wesley Steiner o/a KW Software
	All rights reserved.

***********************************************************************/

/***********************************************************************

Purpose:

	Program to test the PCL parser.

Syntax:

	PCL2TXT <file> [<stdout>]

Input:

	HP PCL file.

Output:

	A formatted listing to stdout of the PCL commands and data
	in the input.

Notes:

$Header:   W:/ljeng/pcl2txt.c_v   1.1   28 Aug 1994  8:50:34   WES  $

$Log:   W:/ljeng/pcl2txt.c_v  $
 * 
 *    Rev 1.1   28 Aug 1994  8:50:34   WES
 *  
 * 
 *    Rev 1.0   28 Aug 1994  8:37:24   WES
 * Initial revision.
 *
 *    Rev 1.0   20 May 1994 14:16:14   WES
 * Initial revision.

***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <string.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys\stat.h>

#include "wes.h"

#include "pcllib.h"

#define VERSION "1.0.0"

int wasdata = 0; /* last byte was a data byte */
int istext = 0; /* currently consuming text */
int isbyte = 0; /* currently consuming bytes of data */
int hexcnt = 0; /* counts # of hex bytes on a line */

char _pchar, _gchar;

int PCL_Hook_ESC(void)
	{
	if (istext || isbyte) printf("\n");
	istext = isbyte = hexcnt = 0;
	return 0;
	}

int PCL_Hook_PCH(const char PCH)
	{
	_pchar = PCH;
	printf("!%c", PCH);
	return 0;
	}

int PCL_Hook_GCH(const char GCH)
	{
	_gchar = GCH;
	printf("%c", GCH);
	return 0;
	}

int PCL_Hook_TCH(const char *Val, const char TCH)
	{
	printf("%s%c", Val, TCH);
	if (isupper(TCH)) printf("\n");
	return 0;
	}

int PCL_Hook_CMD(const char cmdch)

	{
	if (wasdata)
		{
		printf("\n");
		wasdata = 0;
		}
	printf("!%c\n", cmdch);
	return 0;
	}

int PCL_Hook_TXT(const BYTE data)

	{
	wasdata = 1;
	if (
		((!isbyte) && ((data >= 32) && (data <= 126))))
		{
		printf("%c", data);
		istext = 1;
		isbyte = 0;
		}
	else
		{
		if ((istext) || (hexcnt == 16))
			{
			printf("\n");
			hexcnt = 0;
			}
		printf("%02X ", data);
		++hexcnt;
		istext = 0;
		isbyte = 1;
		}
	return 0;
	}

char *UpperCase(char *sz)
	{
	int i;

	for (i = 0; i < strlen(sz); i++) sz[i] = toupper(sz[i]);

	return sz;
	}

int main(int argc, char *argv[])

	{
	int hInput;
	BYTE databyte;
	char fs[128];

	MyAppHeader("PCL2TXT", VERSION, 1994);

	if (argc < 2)
		{
		printf("Syntax: PCL2TXT <file> [<stdout>]\n");
		printf("\n");
		printf("will dissasemble <file>, any HP PCL file, to <stdout> in a readable format.\n");
		printf("\n");
		printf("Ŀ\n");
		printf(" This is a FREEWARE product. You are encouraged to make unmodified copies   \n");
		printf(" of this software to pass on to your friends and associates. Your comments, \n");
		printf(" criticisms and donations are greatly appreciated.                          \n");
		printf("\n");
		printf("\n");
		printf("A registration fee of $50 gets you on the mailing list for updates and other\n");
		printf("products as well as the object code library and ANSI C compatible source code\n");
		printf("for the PCL parser module. The PCL Parser module allows you to incorporate\n");
		printf("HP PCL parsing into your own application(s). The file PCL2TXT.C demonstrates\n");
		printf("how the PCL Parser interfaces with your application.\n");
		printf("\n\n");
		printf("Please send your cheque or money order (Cdn$ in Canada, US$ outside Canada),\n");
		printf("to the address indicated above.\n");
		exit(0);
		}

	strcpy(fs, argv[1]);
	UpperCase(fs);

	if ((hInput = open(fs, O_BINARY | O_RDONLY)) == -1)
		{
		printf("\n%s: File not found or not available.\n", fs);
		exit(0);
		}
	while (read(hInput, &databyte, 1) == 1) PCL_Parse(databyte);
	close(hInput);

	return 0;

	}
